import React from 'react'
import { get } from 'lodash'
import { Button } from '@pubsweet/ui'
import { connect } from 'react-redux'
import styled from 'styled-components'
import { reduxForm } from 'redux-form'
import { th } from '@pubsweet/ui-toolkit'
import { actions } from 'pubsweet-client'
import { withJournal } from 'xpub-journal'
import { ConnectPage } from 'xpub-connect'
import { selectUser } from 'xpub-selectors'
import { create, update } from 'pubsweet-client/src/helpers/api'
import { compose, withProps, withHandlers, withState } from 'recompose'

import AddUserForm from './AddUserForm'
import EditUserForm from './EditUserForm'
import {
  setAdmin,
  getRoleOptions,
  parseUpdateUser,
  handleFormError,
} from './utils'

const onSubmit = (values, dispatch, { isEdit, history }) => {
  if (!isEdit) {
    const newValues = setAdmin(values)
    return create('/users', newValues)
      .then(r => {
        history.push('/admin/users')
        create(`/emails`, {
          email: r.email,
          type: 'invite',
          role: values.role,
        })
      })
      .catch(handleFormError)
  }

  return update(`/users/${values.id}`, parseUpdateUser(values))
    .then(() => {
      history.push('/admin/users')
    })
    .catch(handleFormError)
}

const AddEditUser = ({
  handleSubmit,
  journal,
  isEdit,
  user,
  history,
  error,
  submitting,
}) => (
  <Root>
    <FormContainer onSubmit={handleSubmit}>
      {isEdit ? (
        <EditUserForm
          error={error}
          journal={journal}
          roles={getRoleOptions(journal)}
          subtitle={user.email}
          user={user}
        />
      ) : (
        <AddUserForm
          error={error}
          journal={journal}
          roles={getRoleOptions(journal)}
        />
      )}
      <Row>
        <Button onClick={history.goBack}>Back</Button>
        <Button disabled={submitting} primary type="submit">
          Save user
        </Button>
      </Row>
    </FormContainer>
  </Root>
)

export default compose(
  withJournal,
  withState('isEdit', 'setEdit', false),
  withHandlers({
    setEditMode: ({ setEdit }) => value => setEdit(value),
  }),
  ConnectPage(({ match, setEditMode }) => {
    const id = get(match, 'params.userId')
    const isEditingMode = match.path.includes('/edit/:userId')
    if (isEditingMode && id) {
      setEditMode(isEditingMode)
      return [actions.getUser({ id })]
    }
    return []
  }),
  connect((state, { match }) => ({
    user: selectUser(state, get(match, 'params.userId')),
  })),
  withProps(({ user }) => ({ initialValues: user })),
  reduxForm({
    form: 'userManagement',
    onSubmit,
  }),
)(AddEditUser)

// #region styles

const Root = styled.div`
  display: flex;
  flex-direction: column;
  margin: auto;
  max-width: 60em;
`

const FormContainer = styled.form`
  border: ${th('borderDefault')};
  background-color: ${th('backgroundColorReverse')};
  margin: 0 auto;
  min-width: 300px;
  padding: calc(${th('subGridUnit')} * 3) calc(${th('subGridUnit')} * 6);
`

const Row = styled.div`
  align-items: center;
  display: flex;
  flex-direction: row;
  justify-content: space-evenly;
  margin: ${th('subGridUnit')} 0;
`
// #endregion
