import React from 'react'
import { isEmpty } from 'lodash'
import { connect } from 'react-redux'
import { Button } from '@pubsweet/ui'
import styled from 'styled-components'
import { th } from '@pubsweet/ui-toolkit'
import {
  compose,
  withState,
  lifecycle,
  withHandlers,
  setDisplayName,
} from 'recompose'

import {
  withModal,
  ConfirmationModal,
} from 'pubsweet-component-modal/src/components'

import { Err, Subtitle } from './FormItems'
import { parseSearchParams } from '../utils'
import {
  technicalDecision,
  technicalCheckFetching,
} from '../../redux/technicalCheck'

const EQADecisionPage = ({
  params,
  showEQAModal,
  errorMessage,
  successMessage,
}) => (
  <Root>
    <Title>
      Take a decision for manuscript <b>{params.customId}</b>.
    </Title>
    {errorMessage && <Err>{errorMessage}</Err>}
    {successMessage && <Subtitle>{successMessage}</Subtitle>}
    {isEmpty(errorMessage) &&
      isEmpty(successMessage) && (
        <ButtonContainer>
          <Button onClick={showEQAModal(false)}>RETURN TO EiC</Button>
          <Button onClick={showEQAModal(true)} primary>
            ACCEPT
          </Button>
        </ButtonContainer>
      )}
  </Root>
)

const DeclineModal = compose(
  withState('reason', 'setReason', ''),
  withHandlers({
    changeReason: ({ setReason }) => e => {
      setReason(e.target.value)
    },
  }),
)(({ reason, changeReason, hideModal, onConfirm, modalError }) => (
  <DeclineRoot>
    <span>Return Manuscript to Editor in Chief</span>
    <textarea
      onChange={changeReason}
      placeholder="Return reason*"
      value={reason}
    />
    {modalError && <ErrorMessage>{modalError}</ErrorMessage>}
    <ButtonContainer data-test="eqa-buttons">
      <Button onClick={hideModal}>Cancel</Button>
      <Button disabled={!reason} onClick={onConfirm(reason)} primary>
        Send
      </Button>
    </ButtonContainer>
  </DeclineRoot>
))

const ModalComponent = ({ type, ...rest }) =>
  type === 'decline' ? (
    <DeclineModal {...rest} />
  ) : (
    <ConfirmationModal {...rest} />
  )

export default compose(
  setDisplayName('EQA Decision page'),
  connect(
    state => ({
      isFetching: technicalCheckFetching(state),
    }),
    { technicalDecision },
  ),
  withModal(({ isFetching }) => ({
    isFetching,
    modalComponent: ModalComponent,
  })),
  withState('params', 'setParams', {
    token: null,
    customId: null,
    collectionId: null,
  }),
  withState('successMessage', 'setSuccess', ''),
  lifecycle({
    componentDidMount() {
      const { location, setParams } = this.props
      const { customId, collectionId, token } = parseSearchParams(
        location.search,
      )
      setParams({ customId, collectionId, token })
    },
  }),
  withHandlers({
    showEQAModal: ({
      showModal,
      hideModal,
      setSuccess,
      setModalError,
      technicalDecision,
      params: { collectionId, token },
    }) => decision => () => {
      const acceptConfig = {
        title: `Are you sure you want to accept this EQA package?`,
        onConfirm: () => {
          technicalDecision({
            step: 'eqa',
            agree: decision,
            collectionId,
            token,
          }).then(() => {
            setSuccess(
              `Manuscript accepted. Thank you for your technical check!`,
            )
            hideModal()
          }, setModalError)
        },
        onCancel: hideModal,
      }
      const declineConfig = {
        type: 'decline',
        title: 'Return Manuscript to Editor in Chief',
        onConfirm: reason => () => {
          technicalDecision({
            step: 'eqa',
            agree: decision,
            comments: reason,
            collectionId,
            token,
          }).then(() => {
            setSuccess(
              `Manuscript returned with comments. An email has been sent to Editor In Chief. Thank you for your technical check!`,
            )
            hideModal()
          }, setModalError)
        },
      }

      const cfg = decision ? acceptConfig : declineConfig
      showModal(cfg)
    },
  }),
)(EQADecisionPage)

// #region styles
const Root = styled.div`
  align-items: center;
  color: ${th('colorText')};
  display: flex;
  flex-direction: column;
  justify-content: flex-start;
  margin: 0 auto;
  text-align: center;
  width: 70vw;

  a {
    color: ${th('colorText')};
  }
`

const Title = styled.div`
  color: ${th('colorPrimary')};
  font-size: ${th('fontSizeHeading5')};
  font-family: ${th('fontHeading')};
  margin: 10px auto;
`

const ButtonContainer = styled.div`
  align-items: center;
  display: flex;
  justify-content: space-around;
  padding: calc(${th('gridUnit')} / 2);
  width: calc(${th('gridUnit')} * 15);
`
const ErrorMessage = styled.div`
  color: ${th('colorError')};
  margin: ${th('subGridUnit')};
  text-align: center;
`
const DeclineRoot = styled.div`
  align-items: center;
  background-color: ${th('backgroundColor')};
  display: flex;
  flex-direction: column;
  height: calc(${th('gridUnit')} * 13);
  justify-content: space-between;
  padding: calc(${th('subGridUnit')} * 7);
  width: calc(${th('gridUnit')} * 24);

  & span {
    color: ${th('colorPrimary')};
    font-size: ${th('fontSizeHeading5')};
    font-family: ${th('fontHeading')};
    margin-bottom: ${th('gridUnit')};
  }

  & textarea {
    height: 100%;
    padding: calc(${th('subGridUnit')} * 2);
    width: 100%;
  }

  & textarea:focus,
  & textarea:active {
    outline: none;
  }

  & div {
    display: flex;
    justify-content: space-evenly;
    margin: ${th('gridUnit')} auto 0;
    width: 100%;
  }
`
// #endregion
