import React from 'react'
import { isEmpty } from 'lodash'
import { connect } from 'react-redux'
import { reduxForm } from 'redux-form'
import { required } from 'xpub-validators'
import { Button, H2, TextField, ValidatedField } from '@pubsweet/ui'
import {
  compose,
  withState,
  lifecycle,
  withHandlers,
  setDisplayName,
} from 'recompose'
import {
  Row,
  Text,
  Item,
  Label,
  OpenModal,
  ShadowedBox,
} from 'pubsweet-component-faraday-ui'

import {
  digitValidator,
  parseSearchParams,
  manuscriptIdSizeValidator,
} from '../utils'
import {
  technicalDecision,
  technicalCheckFetching,
} from '../../redux/technicalCheck'

const sizeValidator = manuscriptIdSizeValidator(7)

const Enhanched = () => (
  <Row>
    <Item vertical>
      <Label required>Manuscript ID</Label>
      <ValidatedField
        component={TextField}
        name="manuscriptId"
        validate={[required, digitValidator, sizeValidator]}
      />
    </Item>
  </Row>
)

const FormModal = reduxForm({
  form: 'eqs',
  onSubmit: (values, dispatch, { acceptManuscript }) => {
    acceptManuscript(values)
  },
})(({ isFetching, handleSubmit }) => (
  <OpenModal
    content={Enhanched}
    isFetching={isFetching}
    modalKey="acceptManuscript"
    onConfirm={() => handleSubmit()}
    title="Accept Manuscript"
  >
    {showModal => (
      <Button onClick={showModal} primary>
        ACCEPT
      </Button>
    )}
  </OpenModal>
))

const EQSDecisionPage = ({
  params,
  isFetching,
  eqsDecision,
  errorMessage,
  successMessage,
}) => (
  <ShadowedBox center mt={5}>
    <H2>Editorial decision</H2>
    <Row mt={2}>
      <Text secondary>
        Please take a decision for the manuscript titled <b>{params.title}</b>.
      </Text>
    </Row>
    {successMessage && (
      <Row mt={1}>
        <Text>{successMessage}</Text>
      </Row>
    )}
    {errorMessage && (
      <Row mt={1}>
        <Text error>{errorMessage}</Text>
      </Row>
    )}
    {isEmpty(errorMessage) &&
      isEmpty(successMessage) && (
        <Row justify="space-around" mt={2}>
          <OpenModal
            isFetching={isFetching}
            modalKey="denyManuscript"
            onConfirm={() => {}}
            subtitle="Are you sure you want to reject this manuscript?"
            title="Reject manuscript"
          >
            {showModal => <Button onClick={showModal}>REJECT</Button>}
          </OpenModal>
          <FormModal acceptManuscript={v => {}} isFetching={isFetching} />
        </Row>
      )}
  </ShadowedBox>
)

export default compose(
  setDisplayName('EQS Decision page'),
  connect(
    (state, { params }) => ({
      isFetching: technicalCheckFetching(state),
    }),
    { technicalDecision },
  ),
  withState('params', 'setParams', {
    title: '',
    token: null,
    customId: null,
    collectionId: null,
  }),
  withState('successMessage', 'setSuccess', ''),
  withState('errorMessage', 'setError', ''),
  lifecycle({
    componentDidMount() {
      const { location, setParams } = this.props
      const { customId, collectionId, token, title } = parseSearchParams(
        location.search,
      )
      setParams({ customId, collectionId, token, title })
    },
  }),
  withHandlers({
    eqsDecision: ({
      setError,
      showModal,
      hideModal,
      setSuccess,
      technicalDecision,
      params: { collectionId, token },
    }) => decision => () => {
      showModal({
        title: `Are you sure you want to ${
          decision ? 'accept' : 'reject'
        } this EQS package?`,
        onConfirm: () => {
          technicalDecision({
            step: 'eqs',
            agree: decision,
            collectionId,
            token,
          })
            .then(() => {
              setSuccess(
                `Manuscript ${
                  decision ? 'accepted' : 'rejected'
                }. Thank you for your technical check!`,
              )
              hideModal()
            })
            .catch(() => {
              setError('There was an error. Please try again.')
              hideModal()
            })
        },
        onCancel: hideModal,
      })
    },
  }),
)(EQSDecisionPage)

// #region styles
// #endregion
